<?php
/**
 * Main application class
 *
 * @package wp-checkout-maximizer
 * @author InstaProduct
 * @link http://insta-product.com/
 */

//
//Include all function files
require_once( IPCHM_LIB . '/config.php' );
require_once( IPCHM_LIB . '/setting.php' );
require_once( IPCHM_LIB . '/options.php' );
require_once( IPCHM_LIB . '/font.php' );
require_once( IPCHM_LIB . '/faicon.php' );
require_once( IPCHM_LIB . '/form.php' );
require_once( IPCHM_LIB . '/model.php' );
require_once( IPCHM_APP . '/controller.php' );
require_once( IPCHM_APP . '/viewer.php' );

if ( ! class_exists( 'IPChm_App' ))
{
  class IPChm_App
  {
    //{{ run

    /**
     * Run all functions
     *
     * @return void
     */
    public static function run()
    {
      $hook = sanitize_title( IPCHM_TITLE );

      //
      //Admin init
      add_action( 'admin_init', array( __CLASS__, 'adminInit' ) );

      //
      //Register admin menu
      add_action( 'admin_menu', array( __CLASS__, 'addAdminMenu' ) );

      /*
       * Add meta box option
       */
      add_action( 'add_meta_boxes', array( __CLASS__, 'addMetaBox' ) );
      add_action( 'save_post', array( __CLASS__, 'savePost' ) );

      //
      //Add filter content
      add_filter( 'the_content', array( __CLASS__, 'theContent' ) );
      add_action( 'wp_head', array( __CLASS__, 'ogContent' ) );

      //
      //Shortcode setup
      //

      // Add generator shortcode button
      add_action( 'init', array( __CLASS__, 'addShortcodeEditorButton' ), 100 );
      // Add pop up content
      add_filter( 'the_editor', array( __CLASS__, 'shortcodeEditorFilter' ) );
      add_action( 'admin_footer', array( __CLASS__, 'addShortcodeForm' ) );

      //
      //Add css
      add_action( 'admin_print_styles', array( __CLASS__, 'adminGlobalStyle' ), 1000 );

      //
      //Add javascript
      add_action( 'admin_print_scripts', array( __CLASS__, 'adminGlobalScript' ), 1000 );

      //
      //Front style & script
      add_action('wp_head', array( __CLASS__, 'publicStyle' ) );

      if ( ! is_admin() ) {
        add_action('wp_enqueue_scripts', array( __CLASS__, 'publicScript' ), 10 );
      }
    }

    //}}
    //{{ adminInit

    /**
     * Admin init
     *
     * @return void
     */
    public static function adminInit()
    {
      /**
       * Ajax handler
       **/
      $controller = new IPChm_Controller();
      $controller->ajax();
    }

    //}}
    //{{ addAdminMenu

    /**
     * Add admin menu
     *
     * @return void
     */
    public static function addAdminMenu()
    {
      add_options_page(
    		IPCHM_TITLE,
    		IPCHM_TITLE,
    		'manage_options',
        IPCHM_PAGENAME . '-setting',
        array( __CLASS__, 'setting' )
    	);
    }

    //}}
    //{{ setting

    /**
     * Add administrator setting page
     *
     * @return void
     */
    public static function setting()
    {
      $viewer = new IPChm_Viewer();
      $viewer->setting();
    }

    //}}
    //{{ addMetaBox

    /**
     * Add administrator meta box
     *
     * @return void
     */
    public static function addMetaBox()
    {
        //
        //Add post/page metabox
        $screens = array( 'post', 'page' );
        foreach ($screens as $screen)
        {
            add_meta_box(
                'ipchm_metabox',
                __( 'Checkout Maximizer Settings', IPCHM_SLUG ),
                array( __CLASS__, 'metaBoxContent' ),
                $screen
            );
        }
    }

    //}}
    //{{ metaBoxContent

    /**
     * Content for administrator meta box
     *
     * @return void
     */
    public static function metaBoxContent( $post )
    {
        $viewer = new IPChm_Viewer();
        $viewer->metabox( $post );
    }

    //}}
    //{{ savePost

    /**
     * Save page/post meta box setting
     *
     * @return void
     */
    public static function savePost( $post_id )
    {
        if ( isset( $_POST['post_type'] ) && 'page' == $_POST['post_type'] ) {
            if ( ! current_user_can( 'edit_page', $post_id ) ) {
                return;
            }
        } else {
            if ( ! current_user_can( 'edit_post', $post_id ) ) {
                return;
            }
        }
        // Check if the user intended to change this value.
        if ( ! isset( $_POST['ipchm_mbox_nonce'] ) ) {
            return;
        }
        $post_ID  = $_POST['post_ID'];

        $model = new IPChm_Model();
        $model->saveSetting( $post_ID );
    }

    //}}
    //{{ theContent

    /**
     * Add content upsell to post content
     *
     * @param  string $content
     * @return string
     */
    public static function theContent( $content )
    {
      if ( is_single() || is_page() )
      {
        $viewer   = new IPChm_Viewer();
        $result   = $viewer->content();
        $content .= $result;
      }
      return $content;
    }

    //}}
    //{{ ogContent

    /**
     * Add OG content for share facebook
     *
     * @return string
     */
    public static function ogContent()
    {
      if ( is_single() || is_page() )
      {
        $viewer   = new IPChm_Viewer();
        $result   = $viewer->ogcontent();
        $content .= $result;
      }
      return $content;
    }

    //}}
    //{{ addShortcodeEditorButton

    /**
     * Add shortcode button in content editor
     *
     * @return void
     */
    public static function addShortcodeEditorButton()
    {
      if ( ! current_user_can('edit_posts') && ! current_user_can('edit_pages') ) {
        return;
      }
      if ( get_user_option('rich_editing') == 'true') {
        add_filter( 'mce_external_plugins', array( __CLASS__, 'addTinymcePlugin' ));
        add_filter( 'mce_buttons', array( __CLASS__, 'registerTinymceButton' ));
      }
    }

    //}}
    //{{ addTinymcePlugin

    /**
     * Add tinyMce plugin
     *
     * @return void
     */
    public static function addTinymcePlugin( $plugin_array )
    {
      $plugin_array['ipchm_shortcode'] = IPCHM_BASE_URL . '/assets/js/editor-plugin.js';
      return $plugin_array;
    }

    //}}
    //{{ registerTinymceButton

    /**
     * Add tinyMce button
     *
     * @return void
     */
    public static function registerTinymceButton( $buttons )
    {
      array_push($buttons, "|", "ipchm_shortcode");
      return $buttons;
    }

    //}}
    //{{ shortcodeEditorFilter

    /**
     * The editor filter
     *
     * @param  string $content
     * @return string
     */
    public static function shortcodeEditorFilter( $content )
    {
      global $ipchm_useEditor;
      $ipchm_useEditor = true;
      return $content;
    }

    //}}
    //{{ addShortcodeForm

    /**
     * Add shortcode form
     *
     * @return void
     */
    public static function addShortcodeForm()
    {
      global $ipchm_useEditor;

      wp_enqueue_script('wp-color-picker');
      wp_enqueue_style('wp-color-picker');

      if ( $ipchm_useEditor ) {
        $viewer = new IPChm_Viewer();
        $viewer->shortcodeBackend();
      }
    }

    //}}
    //{{ adminGlobalStyle

    /**
     * Add Css file to all pages
     *
     * @return void
     */
    public static function adminGlobalStyle()
    {
      wp_deregister_style( 'ipchm-admin-global-style' );
      wp_register_style('ipchm-admin-global-style', IPCHM_BASE_URL . 'assets/css/admin-global.min.css', false, null);
      wp_enqueue_style('ipchm-admin-global-style');
    }

    //}}
    //{{ adminGlobalScript

    /**
     * Add javascript file to all pages
     *
     * @return void
     */
    public static function adminGlobalScript()
    {
      ?>
      <script type="text/javascript">
        var ipchm_img_url = '<?php echo IPCHM_BASE_URL . 'assets/img/'; ?>',
            ui_base_image  = '<?php echo IPCHM_BASE_URL . 'assets/img/'; ?>'
      </script>
      <?php
      wp_deregister_script('ipchm-admin-global-script');
      wp_enqueue_script(
        'ipchm-admin-global-script',
        IPCHM_BASE_URL . 'assets/js/admin-global.js',
        array( 'jquery' ),
        1,
        true
      );
    }

    //}}
    //{{ publicStyle

    /**
     * Print public style
     *
     * @return void
     */
    public static function publicStyle()
    {
    ?>
      <script type="text/javascript">
        var ipchm_ajaxurl  = "<?php echo admin_url( 'admin-ajax.php' ); ?>";
        var ipchm_img_url  = '<?php echo IPCHM_BASE_URL . 'assets/img/'; ?>';
      </script>
      <link media="all" type="text/css" href="<?php echo IPCHM_BASE_URL; ?>assets/css/public.min.css" rel="stylesheet">
    <?php
    }

    //}}
    //{{ publicScript

    /**
     * Print public script
     *
     * @return void
     */
    public static function publicScript()
    {
      wp_enqueue_script('jquery');

      wp_deregister_script('jquery-ui');
      wp_register_script('jquery-ui', "http" . ($_SERVER['SERVER_PORT'] == 443 ? "s" : "") . "://ajax.googleapis.com/ajax/libs/jqueryui/1.10.4/jquery-ui.min.js", false, null);
      wp_enqueue_script('jquery-ui');

      wp_deregister_script('ipchm-public-script');
      wp_enqueue_script(
        'ipchm-public-script',
        IPCHM_BASE_URL . 'assets/js/public.js',
        array( 'jquery' ),
        1,
        true
      );
    }

    //}}
  }
}
